#include "bibli.h"

//prof
void display_point(pt p){//python like
  printf("[");
  printf("%4.4f, %4.4f, %4.4f",p.color,p.x,p.y);
  printf("]");
}


/*
p : un point
absmax : abscisse max de l'image
ordmax : ordonnée max de l'image
HYP : le point p est supposé dans les limites de l'image, 
éventuellement sur les bords
RENVOIE : le point de coordonnées entières le plus proche de p
*/
pt proche(pt p, int absmax, int ordmax){
  int x0 = (int) p.x;
  assert (p.x >=0 && p.x <= absmax);
  int x1 = (x0<=absmax)? x0 + 1 : x0;
  int y0 = (int) p.y;
  assert (p.y >=0 && p.y <= ordmax);
  int y1 = (y0<=ordmax)? y0 + 1 : y0;
  int x = (p.x-x0<x1-p.x)? x0:x1;
  int y = (p.y-y0<y1-p.y)? y0:y1;
  pt res = {p.color,x,y};
  return res;
}


//prof
void draw_line(img *im, line *l){
  //printf("draw_line l->card=%d\n",l->card);
  int w=im->width,h=im->height;
  for (int i=0; i< l->card -1; i++)
    trace_segment(im,proche(l->sommets[i],h-1,w-1),
		  proche(l->sommets[i+1],h-1,w-1));
}


/*
p0, p1 : 2 points de même couleur
RENVOIE le point image de p2 par la rotation de centre p0 et d'angle +Pi/3
On rappelle l'existence de M_PI dans math.h
*/
pt third(pt p0, pt p2){
  float pi = M_PI;
  //printf("%f,%f,%f\n",pi,cos(pi/3),sin(pi/3));
  float x = cos(pi/3) * (p2.x - p0.x) - sin(pi/3) * (p2.y - p0.y) + p0.x;
  float y = cos(pi/3) * (p2.y -p0.y) + sin(pi/3) * (p2.x - p0.x) + p0.y;
  pt res = {p0.color,x,y}; 
  return res;
}

void free_line(line * l){
  myfree(l->sommets);
  myfree(l);
}

 
pt _intermediaire(pt A, pt B, float lambda){
  float x = A.x + lambda * (B.x - A.x);
  float y = A.y + lambda * (B.y - A.y);
  pt res = {A.color, x, y};
  return res;
}


/*
l : ligne brisée
Chaque segment A____B de deux points A,B consécutifs dans l

devient une ligne A__/\__B avec 3 points supplémentaires.
Effets de bords : l est modifiée
*/
void maj(line * l){
  int N = l->card;
  pt *sommets = mymalloc( (4*N-3) * sizeof(pt));
  for (int i=0; i < N-1; i++){
    sommets[4*i] = l->sommets[i];
    sommets[4*i+1] = _intermediaire(l->sommets[i],l->sommets[i+1],1./3.);  
    sommets[4*i+3] = _intermediaire(l->sommets[i],l->sommets[i+1],2./3.);
    sommets[4*i+2] = third(sommets[4*i+1],sommets[4*i+3]);
  }
  sommets[4 * (N-1)]=l->sommets[l->card -1];
  myfree(l->sommets);
  l->sommets=sommets;
  l->card=4*N-3;
}

line * koch(int n, pt p0, pt p1){
  assert(n>=0);
  line * l = mymalloc(sizeof(line));
  pt * points = mymalloc(4 * sizeof(pt));
  points[0]=p0;
  points[2]=p1;
  points[1]=third(p0,p1);
  points[3]=p0;
  l->sommets=points;l->card=4;
  while (n!=0){
    maj(l);
    n--;
  }
  return l;
} 

